<#ibiztemplate>
TARGET=PSSYSAPP
</#ibiztemplate>
<#assign isWFAPP = false>
<#if app.getAppMode()?? && app.getAppMode() == "WFAPP" && app.getAllPSAppWFs()??>
    <#assign isWFAPP = true>
</#if>
<#if sys.getAllPSWorkflows()?? && srflist(sys.getAllPSWorkflows())?size gt 0>
package ${pub.getPKGCodeName()}.${app.getPKGCodeName()?lower_case}.job;

import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;
import org.springframework.core.io.ClassPathResource;
import com.alibaba.fastjson.JSONObject;

import java.io.InputStream;
import java.util.List;
import java.util.ArrayList;

import ${pub.getPKGCodeName()}.util.config.SysInfoProperties;
import ${pub.getPKGCodeName()}.util.feign.FeignRequestInterceptor;
import ${pub.getPKGCodeName()}.util.config.UniWFClientProperties;
import ${pub.getPKGCodeName()}.util.feign.UniWFFeignClient;
import ${pub.getPKGCodeName()}.util.domain.*;

/**
 *
 */
@Component
public class WFAppJob implements ApplicationRunner {

    UniWFFeignClient uniWFFeignClient;

    @Autowired
    SysInfoProperties sysInfoProperties;

    @Autowired
    public WFAppJob(Decoder decoder, Encoder encoder, Client client, Contract contract, FeignRequestInterceptor feignRequestInterceptor, UniWFClientProperties uniWFClientProperties) {
        if (uniWFClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(encoder)
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.uniWFFeignClient = nameBuilder.target(UniWFFeignClient.class,"http://" + uniWFClientProperties.getServiceId() + "/") ;
		} else if (uniWFClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(encoder)
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.uniWFFeignClient = nameBuilder.target(UniWFFeignClient.class, "http://" + uniWFClientProperties.getServiceUrl() + "/") ;
		}
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        /* ---------- 工作流应用注册内容 ---------- */

    <#if isWFAPP>
        // 注册工作流应用
        registWFApp();

        // 注册业务工作流版本
        registWFVer();

        // 注册流程处理
        registWFProcess();

        // 注册流程连接
        registWFLink();

        // 注册流程处理角色
        registWFProcRole();

        // 注册嵌套流程
        registPSWFProcSubWF();
    <#else>
        // 注册业务应用
        registPSSysApp();
    </#if>

    }

    <#if isWFAPP>
    /**
     * 注册工作流应用
     */
    private void registWFApp() {
        WFApp wfapp = new WFApp();
        wfapp.setWfappid(sysInfoProperties.getAppid());
        wfapp.setWfappname(sysInfoProperties.getAppname());
        wfapp.setPssystemid(sysInfoProperties.getSystemid());
        wfapp.setWorkflowid(sysInfoProperties.getDomainid());
        wfapp.setServiceid(sysInfoProperties.getServiceid());
        wfapp.setServiceurl(sysInfoProperties.getServiceurl());
        wfapp.setAppurl(sysInfoProperties.getAppurl());
        wfapp.setLogicname(sysInfoProperties.getApplogicname());
        uniWFFeignClient.registWFApp(wfapp);
    }

    /**
     * 注册业务工作流版本
     */
    private void registWFVer() {
        List<PSWFVersion> pswfversions = new ArrayList<PSWFVersion>();
        <#assign i = 0>
        <#list app.getAllPSAppWFVers() as appwfver>
            <#assign appwf = appwfver.getPSAppWF()>
            <#assign wf = appwf.getPSWorkflow()>
            <#assign wfver = appwfver.getPSWFVersion()>
        PSWFVersion pswfversion${i} = new PSWFVersion();
        pswfversion${i}.setPswfversionid("${wfver.getDeployId()}");
        pswfversion${i}.setPswfversionname("${wfver.getName()}");
        pswfversion${i}.setLogicname("${wfver.getName()}");
            <#if wfver.getWFMode()??>
        pswfversion${i}.setWfmodeid("${wfver.getWFMode()}");
            </#if>
        pswfversion${i}.setPsworkflowid("${wf.getDeployId()}");
        pswfversion${i}.setPsworkflowname("${wf.getName()}");
        pswfversion${i}.setWfappid(sysInfoProperties.getAppid());
        pswfversion${i}.setPssystemid(sysInfoProperties.getSystemid());
        pswfversion${i}.setWorkflowid(sysInfoProperties.getDomainid());
        pswfversion${i}.setOrgid(sysInfoProperties.getSysorgid());
        pswfversion${i}.setOrgdeptid(sysInfoProperties.getSysorgdeptid());
            <#if wf.getPSWFDEs()??>
                <#list wf.getPSWFDEs() as pswfde>
                    <#if pswfde.getWFProxyMode() == 2>
        pswfversion${i}.setStarturl(sysInfoProperties.getAppurl() + String.format(sysInfoProperties.getWfstarturl(), "${srfpluralize(pswfde.getPSDataEntity().getCodeName())?lower_case}"));
        pswfversion${i}.setViewurl(sysInfoProperties.getAppurl() + sysInfoProperties.getWfviewurl());
        pswfversion${i}.setModuleid("${pswfde.getPSDataEntity().getCodeName()}");
                        <#break>
                    </#if>
                </#list>
            </#if>
        pswfversion${i}.setVersion(${wfver.getWFVersion()});
        // 文件传入
        ClassPathResource pswfversion${i}DesignFile = new ClassPathResource("workflow/${wfver.getCodeName()}.bpmn");
        InputStream pswfversion${i}Is = null;
        try {
            pswfversion${i}Is = pswfversion${i}DesignFile.getInputStream();
            byte[] pswfversion${i}Content = new byte[pswfversion${i}Is.available()];
            pswfversion${i}Is.read(pswfversion${i}Content);
            pswfversion${i}.setFilename(pswfversion${i}.getPswfversionid() + "${wfver.getCodeName()}.bpmn");
            pswfversion${i}.setFilecontent(pswfversion${i}Content);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (pswfversion${i}Is != null) {
                try {
                    pswfversion${i}Is.close();
                } catch (Exception e2) {
                    e2.printStackTrace();
                }
            }
            pswfversion${i}Is = null;
        }
        pswfversions.add(pswfversion${i});
            <#assign i++>
        </#list>
        uniWFFeignClient.registPSWFVersion(pswfversions);
    }

    /**
     * 注册流程处理
     */
    private void registWFProcess() {
        List<PSWFProcess> pswfprocesses = new ArrayList<PSWFProcess>();
        <#assign i = 0>
        <#list app.getAllPSAppWFVers() as appwfver>
            <#assign appwf = appwfver.getPSAppWF()>
            <#assign wf = appwf.getPSWorkflow()>
            <#assign wfver = appwfver.getPSWFVersion()>
            <#if wfver.getPSWFProcesses()??>
                <#list wfver.getPSWFProcesses() as wfproc>
        PSWFProcess pswfprocess${i} = new PSWFProcess();
        pswfprocess${i}.setPswfprocessid("${wfproc.getDeployId()}");
        pswfprocess${i}.setPswfprocessname("${wfproc.getName()}");
        pswfprocess${i}.setPswfversionid("${wfver.getDeployId()}");
        pswfprocess${i}.setPswfversionname("${wfver.getName()}");
            <#if wfproc.getTargetPSWF??>
        pswfprocess${i}.setTargetpswfid("${wfproc.getTargetPSWF().getDeployId()}");
        pswfprocess${i}.setTargetpswfname("${wfproc.getTargetPSWF().getName()}");
            </#if>
<#--    模型暂无    -->
<#--        pswfprocess${i}.setPredefinedactions("");-->
        pswfprocess${i}.setWfprocesstype("${wfproc.getWFProcessType()}");
        pswfprocess${i}.setWfstepvalue("${wfproc.getWFStepValue()}");
<#--    步骤名称代码表值，暂时无法取到    -->
<#--        pswfprocess${i}.setWfstepname("${wfproc.getWFStepValue()}");-->
            <#if wfproc.getMultiInstMode??>
        pswfprocess${i}.setMultiinstmode("${wfproc.getMultiInstMode()}");
            </#if>
        pswfprocesses.add(pswfprocess${i});
                    <#assign i++>
                </#list>
            </#if>
        </#list>
        uniWFFeignClient.registPSWFProcess(pswfprocesses);
    }

    /**
     * 注册流程连接
     */
    private void registWFLink() {
        List<PSWFLink> pswflinks = new ArrayList<PSWFLink>();
        <#assign i = 0>
        <#list app.getAllPSAppWFVers() as appwfver>
            <#assign appwf = appwfver.getPSAppWF()>
            <#assign wf = appwf.getPSWorkflow()>
            <#assign wfver = appwfver.getPSWFVersion()>
            <#if wfver.getPSWFLinks()??>
                <#list wfver.getPSWFLinks() as wflink>
        PSWFLink pswflink${i} = new PSWFLink();
        pswflink${i}.setPswflinkid("${wflink.getDeployId()}");
        pswflink${i}.setPswflinkname("${wflink.getName()}");
        pswflink${i}.setLogicname("${wflink.getLogicName()}");
        pswflink${i}.setFrompswfprocid("${wflink.getFromPSWFProcess().getDeployId()}");
        pswflink${i}.setTopswfprocid("${wflink.getToPSWFProcess().getDeployId()}");
        pswflink${i}.setPswfversionid("${wfver.getDeployId()}");
        pswflink${i}.setPswfversionname("${wfver.getName()}");
        pswflinks.add(pswflink${i});
                    <#assign i++>
                </#list>
            </#if>
        </#list>
        uniWFFeignClient.registPSWFLink(pswflinks);
    }

    /**
     * 注册流程处理角色
     */
    private void registWFProcRole() {
        List<PSWFProcRole> pswfprocroles = new ArrayList<PSWFProcRole>();
        <#assign i = 0>
        <#assign j = 0>
        <#list app.getAllPSAppWFVers() as appwfver>
            <#assign appwf = appwfver.getPSAppWF()>
            <#assign wfver = appwfver.getPSWFVersion()>
            <#if wfver.getPSWFProcesses()??>
                <#list wfver.getPSWFProcesses() as wfproc>
        JSONObject data${j} = new JSONObject();
        data${j}.put("pswfprocessid", "${wfproc.getDeployId()}");
        uniWFFeignClient.clearPSWFProcRole(data${j});
                    <#if wfproc.getPSWFProcessRoles?? && wfproc.getPSWFProcessRoles()??>
                        <#list wfproc.getPSWFProcessRoles() as wfprocrole>
        PSWFProcRole pswfprocrole${i} = new PSWFProcRole();
        pswfprocrole${i}.setPswfprocessid("${wfproc.getDeployId()}");
        pswfprocrole${i}.setPswfprocessname("${wfproc.getName()}");
        pswfprocrole${i}.setPswfprocroleid("${wfprocrole.getDeployId()}");
        pswfprocrole${i}.setPswfprocrolename("${wfprocrole.getName()}");
            <#if wfprocrole.getPSWFRole()??>
                <#assign wfrole = wfprocrole.getPSWFRole()>
        pswfprocrole${i}.setPswfroleid("${wfrole.getDeployId()}");
        pswfprocrole${i}.setPswfrolename("${wfrole.getName()}");
        </#if>
            <#if wfprocrole.getUDField()??>
        pswfprocrole${i}.setPswfprocrolename("${wfprocrole.getName()}${wfprocrole.getUDField()}");
        pswfprocrole${i}.setUdfield("${wfprocrole.getUDField()}");
            </#if>
        pswfprocrole${i}.setRoletype("${wfprocrole.getWFProcessRoleType()}");
        pswfprocroles.add(pswfprocrole${i});
                    <#assign i++>
                        </#list>
                    </#if>
                    <#assign j++>
                </#list>
            </#if>
        </#list>
        uniWFFeignClient.registPSWFProcRole(pswfprocroles);
    }

    /**
     * 注册嵌套流程
     */
    private void registPSWFProcSubWF() {
        List<PSWFProcSubWF> pswfprocsubwfs = new ArrayList<PSWFProcSubWF>();
        <#assign i = 0>
        <#assign j = 0>
        <#list app.getAllPSAppWFVers() as appwfver>
            <#assign appwf = appwfver.getPSAppWF()>
            <#assign wfver = appwfver.getPSWFVersion()>
            <#if wfver.getPSWFProcesses()??>
                <#list wfver.getPSWFProcesses() as wfproc>
        JSONObject data${j} = new JSONObject();
        data${j}.put("pswfprocessid", "${wfproc.getDeployId()}");
        uniWFFeignClient.clearPSWFProcSubWF(data${j});
                <#if wfproc.getPSWFProcessSubWFs?? && wfproc.getPSWFProcessSubWFs()??>
                    <#list wfproc.getPSWFProcessSubWFs() as subwf>
        PSWFProcSubWF pswfprocsubwf${i} = new PSWFProcSubWF();
        pswfprocsubwf${i}.setPswfprocsubwfname("${subwf.getName()}");
        pswfprocsubwf${i}.setPswfprocsubwfid("${subwf.getDeployId()}");
        pswfprocsubwf${i}.setPswfversionid("${subwf.getPSWFVersion().getDeployId()}");
        pswfprocsubwf${i}.setPswfprocessid("${wfproc.getDeployId()}");
        pswfprocsubwf${i}.setPsworkflowid("${subwf.getPSWorkflow().getDeployId()}");
        pswfprocsubwf${i}.setSubwfdeid("${subwf.getPSDataEntity().getDeployId()}");
        pswfprocsubwf${i}.setSubwfdedataset("${subwf.getPSDEDataSet().getCodeName()}");
        pswfprocsubwfs.add(pswfprocsubwf${i});
                    <#assign i++>
                        </#list>
                    </#if>
                    <#assign j++>
                </#list>
            </#if>
        </#list>
        uniWFFeignClient.registPSWFProcSubWF(pswfprocsubwfs);
    }
    <#else>
    /**
     * 注册业务应用
     */
    private void registPSSysApp() {
        PSSysApp pssysapp = new PSSysApp();
        pssysapp.setPssysappid(sysInfoProperties.getAppid());
        pssysapp.setPssysappname(sysInfoProperties.getAppname());
        pssysapp.setPssystemid(sysInfoProperties.getSystemid());
        pssysapp.setWorkflowid(sysInfoProperties.getDomainid());
        pssysapp.setServiceid(sysInfoProperties.getServiceid());
        pssysapp.setServiceurl(sysInfoProperties.getServiceurl());
        pssysapp.setAppurl(sysInfoProperties.getAppurl());
        pssysapp.setProxyviewurl(sysInfoProperties.getProxyviewurl());
        pssysapp.setLogicname(sysInfoProperties.getApplogicname());
        uniWFFeignClient.registPSSysApp(pssysapp);
    }
    </#if>
}
</#if>
